@@ -4,6 +4,7 @@
 #include "soil.h"
 #include "root.h"
 #include "hydraulics.h"
+#include "communication_structures.h"
 #include "biophysicsutils.h"
 #include "numerical_solving.h"
 #include <meteoland.h>
@@ -420,7 +421,7 @@ double microporeImbibitionRate(double theta_b, double theta_micro,
   double gamma_w = 0.1; //Scaling factor
   double D_w = ((D_theta_b + D_theta_micro)/2.0)*S_macro;//Effective water diffusivity
   double d = 9.35*1e-3; //Effective diffusion pathlength in m
-  double S_w = std::max(0.0, ((G_f*D_w*gamma_w)/pow(d, 2.0))*(theta_b - theta_micro));
+  double S_w = std::max(0.0, ((G_f*D_w*gamma_w)/(d*d))*(theta_b - theta_micro));
   // Rcout<< "D_w " << D_w << " S_w "<< S_w<<"\n";
   return(S_w);
 }
@@ -479,116 +480,12 @@ double rootFindingMacropores(double S_t, double K_up, double Ksat_ms, double Ksa
   return(c);
 }
 
-//' Soil water balance
-//' 
-//' Function \code{hydrology_soilWaterBalance} estimates water balance of soil layers given water inputs/outputs, including the simulation of water movement within the soil.
-//' 
-//' @param soil Object of class \code{\link{soil}}.
-//' @param soilFunctions Soil water retention curve and conductivity functions, either 'SX' (for Saxton) or 'VG' (for Van Genuchten).
-//' @param rainfallInput Amount of water from rainfall event (after excluding interception), in mm.
-//' @param rainfallIntensity Rainfall intensity, in mm/h.
-//' @param snowmelt Amount of water originated from snow melt, in mm.
-//' @param sourceSink Local source/sink term for each soil layer (from soil evaporation or plant transpiration/redistribution)
-//'        as mm/day.
-//' @param runon Surface water amount running on the target area from upslope (in mm).
-//' @param lateralFlows Lateral source/sink terms for each soil layer (interflow/to from adjacent locations) as mm/day.
-//' @param waterTableDepth Water table depth (in mm). When not missing, capillarity rise will be allowed if lower than total soil depth.
-//' @param infiltrationMode Infiltration model, either "GreenAmpt1911" or "Boughton1989"
-//' @param infiltrationCorrection Correction for saturated conductivity, to account for increased infiltration due to macropore presence
-//' @param soilDomains Either "buckets" (multi-bucket domain), "single" (for single-domain Richards) or "dual" (for dual-permeability model).
-//' @param nsteps Number of time steps per day
-//' @param max_nsubsteps Maximum number of substeps per time step
-//' @param modifySoil Boolean flag to indicate that the input \code{soil} object should be modified during the simulation.
-//' 
-//' @seealso  \code{\link{spwb}}, \code{\link{hydrology_waterInputs}}, \code{\link{hydrology_infiltration}}
-//' 
-//' @details
-//' The multi-bucket model adds/substracts water to each layer and if content is above field capacity the excess percolates to the layer below.
-//' If there is still an excess for the bottom layer, the model will progressively fill upper layers (generating saturation excess if the first layer 
-//' becomes saturated). Every day the some layers are over field capacity, the model simulates deep drainage.
-//' 
-//' The single-domain model simulates water flows by solving Richards's equation using the predictor-corrector method, as described in 
-//' Bonan et al. (2019).
-//' 
-//' The dual-permeability model is an implementation of the model MACRO 5.0 (Jarvis et al. 1991; Larsbo et al. 2005).
-//' 
-//' Both the multi-bucket and the single-domain model can apply a correction to the infiltration rate to account for macroporosity in infiltration. In the
-//' dual-permeability model extra infiltration through macropores is simulated explicitly.
-//' 
-//' @author 
-//' Miquel De \enc{Cáceres}{Caceres} Ainsa, CREAF
-//' 
-//' \enc{María González Sanchís}{Maria Gonzalez Sanchis}, UPV-CTFC
-//' 
-//' @return
-//'   Returns a named vector with different elements, depending on \code{soilDomains}. 
-//'   
-//'   If \code{soilDomains == "buckets"}:
-//'   \itemize{
-//'     \item{\code{Snowmelt}: Snowmelt input (mm).}
-//'     \item{\code{Source/sinks}: Sum of source/sink input across layers (mm).}
-//'     \item{\code{Infiltration}: Water infiltrated into the soil (mm).}
-//'     \item{\code{InfiltrationExcess}: Excess infiltration in the topmost layer (mm) leading to an increase in runoff.}
-//'     \item{\code{SaturationExcess}: Excess saturation in the topmost layer (mm) leading to an increase in runoff.}
-//'     \item{\code{Runoff}: Surface runoff generated by saturation excess or infiltration excess (mm).}
-//'     \item{\code{DeepDrainage}: Water draining from the bottom layer (mm). This quantity is corrected to close the water balance.}
-//'     \item{\code{CapillarityRise}: Water entering the soil via capillarity rise (mm) from the water table, if \code{waterTableDepth} is supplied.}
-//'   }
-//'   If \code{soilDomains == "single"} the named vector contains the following additional elements:
-//'   \itemize{
-//'     \item{\code{Correction}: Amount of water (mm) added to deep drainage to correct the water balance.}
-//'     \item{\code{VolumeChange}: Change in soil water volume (mm).}
-//'     \item{\code{Substep}: Time step of the moisture solving (seconds).}
-//'   }
-//'  If \code{soilDomains == "dual"} the named vector contains the following additional elements:
-//'   \itemize{
-//'     \item{\code{Lateral flows}: Sum of water circulating between micropores and macropores, positive when filling micropores (mm).}
-//'     \item{\code{InfiltrationMatrix}: Water infiltrated into the soil matrix (mm).}
-//'     \item{\code{InfiltrationMacropores}: Water infiltrated into the soil macropore domain (mm).}
-//'     \item{\code{InfiltrationExcessMatrix/InfiltrationExcessMacropores}: Excess infiltration in the topmost layer (mm) leading to an increase in runoff.}
-//'     \item{\code{SaturationExcessMatrix/SaturationExcessMacropores}: Excess saturation in the topmost layer (mm) leading to an increase in runoff.}
-//'     \item{\code{DrainageMatrix}: Water draining from the bottom layer of the matrix domain (mm). This quantity is corrected to close water balance in the micropore domain.}
-//'     \item{\code{DrainageMacropores}: Water draining from the bottom layer of the macropore domain (mm). This quantity is corrected to close the water balance in the macropore domain.}
-//'     \item{\code{CorrectionMatrix}: Amount of water (mm) added to deep drainage of soil matrix to correct the water balance.}
-//'     \item{\code{CorrectionMacropores}: Amount of water (mm) added to deep drainage of macropores to correct the water balance.}
-//'     \item{\code{MatrixVolumeChange}: Change in soil water volume in the soil matrix domain (mm).}
-//'     \item{\code{MacroporeVolumeChange}: Change in soil water volume in the macropore domain (mm).}
-//'   }
-//' 
-//' @references
-//' Bonan, G. (2019). Climate change and terrestrial ecosystem modeling. Cambridge University Press, Cambridge, UK. 
-//' 
-//' Jarvis, N.J., Jansson, P‐E., Dik, P.E. & Messing, I. (1991). Modelling water and solute transport in macroporous soil. I. Model description and sensitivity analysis. Journal of Soil Science, 42, 59–70. 
-//' 
-//' Larsbo, M., Roulier, S., Stenemo, F., Kasteel, R. & Jarvis, N. (2005). An Improved Dual‐Permeability Model of Water Flow and Solute Transport in the Vadose Zone. Vadose Zone Journal, 4, 398–406. 
-//' 
-//' @examples
-//' # Define soil parameters
-//' spar <- defaultSoilParams(4)
-//' 
-//' # Initializes soil hydraulic parameters
-//' examplesoil <- soil(spar)
-//' 
-//' # Water balance in a multi-bucket model
-//' hydrology_soilWaterBalance(examplesoil, "VG", 10, 5, 0, c(-1,-1,-1,-1), 
-//'                            soilDomains = "buckets", modifySoil = FALSE)
-//'                            
-//' # Water balance in a single-domain model (Richards equation)
-//' hydrology_soilWaterBalance(examplesoil, "VG", 10, 5, 0, c(-1,-1,-1,-1), 
-//'                            soilDomains = "single", modifySoil = FALSE)
-//'                     
-//' # Water balance in a dual-permeability model (MACRO)
-//' hydrology_soilWaterBalance(examplesoil, "VG", 10, 5, 0, c(-1,-1,-1,-1), 
-//'                            soilDomains = "dual", modifySoil = FALSE)
-//'   
-//' @name hydrology_soilWaterBalance
-//' @keywords internal
-// [[Rcpp::export("hydrology_soilWaterBalance")]]
-NumericVector soilWaterBalance(DataFrame soil, String soilFunctions, 
-                               double rainfallInput, double rainfallIntensity, double snowmelt, NumericVector sourceSink, 
-                               double runon = 0.0, Nullable<NumericVector> lateralFlows = R_NilValue, double waterTableDepth = NA_REAL,
-                               String infiltrationMode = "GreenAmpt1911", double infiltrationCorrection = 5.0, String soilDomains = "buckets", 
-                               int nsteps = 24, int max_nsubsteps = 3600, bool modifySoil = true) {
+
+NumericVector soilWaterBalance_inner(List SWBcommunication, DataFrame soil, String soilFunctions, 
+                                     double rainfallInput, double rainfallIntensity, double snowmelt, NumericVector sourceSink, 
+                                     double runon = 0.0, Nullable<NumericVector> lateralFlows = R_NilValue, double waterTableDepth = NA_REAL,
+                                     String infiltrationMode = "GreenAmpt1911", double infiltrationCorrection = 5.0, String soilDomains = "buckets", 
+                                     int nsteps = 24, int max_nsubsteps = 3600, bool modifySoil = true) {
   
   if((soilDomains!="single") && (soilDomains!="dual")  && (soilDomains!="buckets") ) stop("Unrecognized soilDomain value");
   if(!modifySoil) soil = clone(soil);
@@ -597,7 +494,7 @@ NumericVector soilWaterBalance(DataFrame soil, String soilFunctions,
   NumericVector macro = soil["macro"];
   NumericVector rfc = soil["rfc"];
   NumericVector W = soil["W"];
-  int nlayers = widths.size();
+  int nlayers = soil.nrow();
   
   bool micropore_imbibition = true;
   
@@ -618,7 +515,7 @@ NumericVector soilWaterBalance(DataFrame soil, String soilFunctions,
   double macropore_correction_mm = 0.0;
   
   
- 
+  
   //Add snow-melt and runon to infiltration_matrix_mm
   double snowmelt_mm = 0.0;
   if(!NumericVector::is_na(snowmelt)) {
@@ -654,7 +551,7 @@ NumericVector soilWaterBalance(DataFrame soil, String soilFunctions,
       source_sink_def_mm[l] += lateralFlows_mm[l];
     }
   }
-
+  
   NumericVector res;
   if(soilDomains == "buckets") {
     NumericVector Ksat = soil["Ksat"];
@@ -739,11 +636,16 @@ NumericVector soilWaterBalance(DataFrame soil, String soilFunctions,
     NumericVector matrixExcess_m3s(nlayers, 0.0);
     NumericVector saturated_matrix_correction_m3s(nlayers, 0.0);
     NumericVector saturated_macropore_correction_m3s(nlayers, 0.0);
-    NumericVector dZ_m = widths*0.001; //mm to m
     
     
+    NumericVector dZ_m = SWBcommunication[SOILCOM_dZ_m];
+    for(int l=0;l<nlayers;l++) dZ_m[l] = widths[l]*0.001; //mm to m
+    
+    NumericVector dZUp = SWBcommunication[SOILCOM_dZUp];
+    NumericVector dZDown = SWBcommunication[SOILCOM_dZDown];
+    NumericVector lambda = SWBcommunication[SOILCOM_lambda];
+    
     //Estimate layer interfaces
-    NumericVector dZUp(nlayers), dZDown(nlayers), lambda(nlayers);
     for(int l=0;l<nlayers;l++) {
       lambda[l] = 1.0 - (rfc[l]/100.0);
       if(l==0) { //first layer
@@ -791,15 +693,58 @@ NumericVector soilWaterBalance(DataFrame soil, String soilFunctions,
     //Estimate Theta, Psi, C, K
     NumericVector Theta = theta(soil, "VG");
     NumericVector Theta_FC = thetaFC(soil, "VG");
+    
     //Microporosity or single domain
-    NumericVector theta_micro(nlayers), theta_b(nlayers), theta_macro(nlayers), theta_sat_fict(nlayers), Ksat_b(nlayers), Ksat_b_ms(nlayers);
-    NumericVector Ksat(nlayers), Ksat_ms(nlayers);
-    NumericVector Psi(nlayers), K(nlayers), C(nlayers);
-    NumericVector Psi_m(nlayers), K_ms(nlayers), Kbc(nlayers), Kbc_ms(nlayers), C_m(nlayers);
-    //Macroporosoty domain
-    NumericVector S_macro(nlayers), e_macro(nlayers), Kmacro_ms(nlayers);
-    NumericVector waterFluidity(nlayers, 1.0);
+    NumericVector theta_micro = SWBcommunication[SOILCOM_theta_micro];
+    NumericVector theta_b = SWBcommunication[SOILCOM_theta_b];
+    NumericVector theta_macro = SWBcommunication[SOILCOM_theta_macro];
+    NumericVector theta_sat_fict = SWBcommunication[SOILCOM_theta_sat_fict];
+    NumericVector Ksat_b = SWBcommunication[SOILCOM_Ksat_b];
+    NumericVector Ksat_b_ms = SWBcommunication[SOILCOM_Ksat_b_ms];
+    NumericVector Ksat = SWBcommunication[SOILCOM_Ksat];
+    NumericVector Ksat_ms = SWBcommunication[SOILCOM_Ksat_ms];
+    NumericVector Psi = SWBcommunication[SOILCOM_Psi];
+    NumericVector K = SWBcommunication[SOILCOM_K];
+    NumericVector C = SWBcommunication[SOILCOM_C];
+    NumericVector Psi_m = SWBcommunication[SOILCOM_Psi_m];
+    NumericVector K_ms = SWBcommunication[SOILCOM_K_ms];
+    NumericVector Kbc = SWBcommunication[SOILCOM_Kbc];
+    NumericVector Kbc_ms = SWBcommunication[SOILCOM_Kbc_ms];
+    NumericVector C_m = SWBcommunication[SOILCOM_C_m];
     for(int l=0;l<nlayers;l++) {
+      theta_micro[l] = 0.0;
+      theta_b[l] = 0.0;
+      theta_macro[l] = 0.0;
+      theta_sat_fict[l] = 0.0;
+      Ksat_b[l] = 0.0;
+      Ksat_b_ms[l] = 0.0;
+      Ksat[l] = 0.0;
+      Psi[l] = 0.0;
+      K[l] = 0.0;
+      C[l] = 0.0;
+      Psi_m[l] = 0.0;
+      K_ms[l] = 0.0;
+      Kbc[l] = 0.0;
+      Kbc_ms[l] = 0.0;
+      C_m[l] = 0.0;
+    }
+    
+    //Macroporosity domain
+    NumericVector S_macro, e_macro, Kmacro_ms;
+    if(soilDomains=="dual") {
+      S_macro = SWBcommunication[SOILCOM_S_macro];
+      e_macro = SWBcommunication[SOILCOM_e_macro];
+      Kmacro_ms = SWBcommunication[SOILCOM_Kmacro_ms];
+      for(int l=0;l<nlayers;l++) {
+        S_macro[l] = 0.0;
+        e_macro[l] = 0.0;
+        Kmacro_ms[l] = 0.0;
+      }
+    }
+    
+    NumericVector waterFluidity = SWBcommunication[SOILCOM_waterFluidity];
+    for(int l=0;l<nlayers;l++) {
+      waterFluidity[l] = 1.0;
       if(!NumericVector::is_na(Tsoil[l])) {
         if(Tsoil[l]>0) {
           waterFluidity[l] = 1.0/waterDynamicViscosity(Tsoil[l]); 
@@ -877,15 +822,54 @@ NumericVector soilWaterBalance(DataFrame soil, String soilFunctions,
     double saturation_excess_macropores_mm = 0.0;
     double K_up= 0.0, K_down= 0.0;
     
-    NumericVector a(nlayers, 0.0), b(nlayers, 0.0), c(nlayers, 0.0), d(nlayers, 0.0);
-    NumericVector C_step(nlayers, 0.0), C_step_m(nlayers, 0.0), K_step_ms(nlayers, 0.0), K_step(nlayers, 0.0), Psi_step(nlayers, 0.0), Psi_step_m(nlayers, 0.0);
-    NumericVector S_macro_step(nlayers, 0.0), Kmacro_step_ms(nlayers, 0.0), theta_macro_step(nlayers, 0.0), theta_micro_step(nlayers, 0.0);
     double C_step_05, K_step_05;
-    NumericVector K_step_ms05(nlayers, 0.0), C_step_m05(nlayers, 0.0);
-    NumericVector finalSourceSinks_m3s(nlayers, 0.0);
-    NumericVector capill_below(nlayers, 0.0);
-    NumericVector drain_above(nlayers, 0.0);
-    NumericVector drain_below(nlayers, 0.0);
+    
+    NumericVector a = SWBcommunication[SOILCOM_a];
+    NumericVector b = SWBcommunication[SOILCOM_b];
+    NumericVector c = SWBcommunication[SOILCOM_c];
+    NumericVector d = SWBcommunication[SOILCOM_d];
+    NumericVector e = SWBcommunication[SOILCOM_e];
+    NumericVector f = SWBcommunication[SOILCOM_f];
+    
+    NumericVector K_step_ms05 = SWBcommunication[SOILCOM_K_step_ms05];
+    NumericVector C_step_m05 = SWBcommunication[SOILCOM_C_step_m05];
+    NumericVector C_step = SWBcommunication[SOILCOM_C_step];
+    NumericVector C_step_m = SWBcommunication[SOILCOM_C_step_m];
+    NumericVector K_step_ms = SWBcommunication[SOILCOM_K_step_ms];
+    NumericVector K_step = SWBcommunication[SOILCOM_K_step];
+    NumericVector Psi_step = SWBcommunication[SOILCOM_Psi_step];
+    NumericVector Psi_step_m = SWBcommunication[SOILCOM_Psi_step_m];
+    NumericVector S_macro_step = SWBcommunication[SOILCOM_S_macro_step];
+    NumericVector Kmacro_step_ms = SWBcommunication[SOILCOM_Kmacro_step_ms];
+    NumericVector theta_macro_step = SWBcommunication[SOILCOM_theta_macro_step];
+    NumericVector theta_micro_step = SWBcommunication[SOILCOM_theta_micro_step];
+    for(int l=0;l<nlayers;l++) {
+      K_step_ms05[l] = 0.0;
+      C_step_m05[l] = 0.0;
+      C_step[l] = 0.0;
+      C_step_m[l] = 0.0;
+      K_step_ms[l] = 0.0;
+      K_step[l] = 0.0;
+      Psi_step[l] = 0.0;
+      Psi_step_m[l] = 0.0;
+      S_macro_step[l] = 0.0;
+      Kmacro_step_ms[l] = 0.0;
+      theta_macro_step[l] = 0.0;
+      theta_micro_step[l] = 0.0;
+    }
+    NumericVector finalSourceSinks_m3s = SWBcommunication[SOILCOM_finalSourceSinks_m3s];
+    NumericVector capill_below = SWBcommunication[SOILCOM_capill_below];
+    NumericVector drain_above = SWBcommunication[SOILCOM_drain_above];
+    NumericVector drain_below = SWBcommunication[SOILCOM_drain_below];
+    NumericVector lateral_flows_step_mm = SWBcommunication[SOILCOM_lateral_flows_step_mm];
+    for(int l=0;l<nlayers;l++) {
+      finalSourceSinks_m3s[l] = 0.0;
+      capill_below[l] = 0.0;
+      drain_above[l] = 0.0;
+      drain_below[l] = 0.0;
+      lateral_flows_step_mm[l] = 0.0;
+    }
+    
     double drainage_matrix_step_m3 = 0.0;
     double drainage_macropores_step_m3 = 0.0;
     double capillarity_matrix_step_m3 = 0.0;
@@ -898,7 +882,6 @@ NumericVector soilWaterBalance(DataFrame soil, String soilFunctions,
     double macropore_correction_step_mm = 0.0;
     double infiltration_remaining_macropores_step_mm = 0.0; 
     double infiltration_target_macropores_step_mm = 0.0;
-    NumericVector lateral_flows_step_mm(nlayers, 0.0);
     
     int total_nsubsteps = 0;
     int nsubsteps = 1;
@@ -934,9 +917,11 @@ NumericVector soilWaterBalance(DataFrame soil, String soilFunctions,
           K_step_ms[l] = K_ms[l];
           Psi_step[l] = Psi[l];
           Psi_step_m[l] = Psi_m[l];
-          S_macro_step[l] = S_macro[l];
-          Kmacro_step_ms[l] = Kmacro_ms[l];
-          theta_macro_step[l] = theta_macro[l];
+          if(soilDomains=="dual") {
+            S_macro_step[l] = S_macro[l];
+            Kmacro_step_ms[l] = Kmacro_ms[l];
+            theta_macro_step[l] = theta_macro[l];
+          }
           theta_micro_step[l] = theta_micro[l];
         }
         
@@ -1049,8 +1034,11 @@ NumericVector soilWaterBalance(DataFrame soil, String soilFunctions,
               d[l] = (lambda[l]*C_step_m[l]*dZ_m[l]/halftsubstep)*Psi_step_m[l] + drain_above[l] - drain_below[l] + capill_below[l] + finalSourceSinks_m3s[l];
             }
           }
-          NumericVector Psi_step_m_t05 = tridiagonalSolving(a,b,c,d);
-          NumericVector Psi_step_t05 = Psi_step_m_t05*mTOMPa; // m to MPa
+          //TRIDIAGONAL SOLVING
+          NumericVector Psi_step_t05 = tridiagonalSolving(a,b,c,d,e,f, nlayers);
+          //MODIFY UNITS OF OUTPUT PSI
+          for(int l=0;l<nlayers;l++) Psi_step_t05[l] = Psi_step_t05[l]*mTOMPa; // m to MPa
+          
           //Calculate K and C at t05
           for(int l=0;l<nlayers;l++) {
             if(soilDomains=="single") {
@@ -1116,8 +1104,11 @@ NumericVector soilWaterBalance(DataFrame soil, String soilFunctions,
               d[l] = (lambda[l]*C_step_m05[l]*dZ_m[l]/tsubstep)*Psi_step_m[l] + a[l]*(Psi_step_m[l - 1] - Psi_step_m[l]) + capill_below[l] + drain_above[l] - drain_below[l] + finalSourceSinks_m3s[l];
             }
           }
-          NumericVector Psi_step_m_t1 = tridiagonalSolving(a,b,c,d);
-          NumericVector Psi_step_t1 = Psi_step_m_t1*mTOMPa; // m to MPa
+          //TRIDIAGONAL SOLVING
+          NumericVector Psi_step_t1 = tridiagonalSolving(a,b,c,d, e, f, nlayers);
+          //MODIFY UNITS OF OUTPUT PSI
+          for(int l=0;l<nlayers;l++) Psi_step_t1[l] = Psi_step_t1[l]*mTOMPa; // m to MPa
+          
           //calculate drainage (m3)
           if(freeDrainage) {
             drainage_matrix_step_m3 += drain_below[nlayers -1]*tsubstep;
@@ -1329,8 +1320,10 @@ NumericVector soilWaterBalance(DataFrame soil, String soilFunctions,
         } else {
           //Add remaining target to infiltration excess
           infiltration_excess_macropores_step_mm = infiltration_remaining_macropores_step_mm;
-          double balance_micro_step_mm = m3_2_mm*(tstep*sum(source_sink_def_m3s) + capillarity_matrix_step_m3 - drainage_matrix_step_m3) + sum(lateral_flows_step_mm) - saturation_excess_matrix_step_mm;
-          double balance_macro_step_mm = infiltration_macropores_step_mm + m3_2_mm*(capillarity_macropores_step_m3 - drainage_macropores_step_m3) - sum(lateral_flows_step_mm) - saturation_excess_macropores_step_mm;
+          double sum_lateral_flows_step_mm = 0.0;
+          for(int l=0;l<nlayers;l++) sum_lateral_flows_step_mm += lateral_flows_step_mm[l];
+          double balance_micro_step_mm = m3_2_mm*(tstep*sum(source_sink_def_m3s) + capillarity_matrix_step_m3 - drainage_matrix_step_m3) + sum_lateral_flows_step_mm - saturation_excess_matrix_step_mm;
+          double balance_macro_step_mm = infiltration_macropores_step_mm + m3_2_mm*(capillarity_macropores_step_m3 - drainage_macropores_step_m3) - sum_lateral_flows_step_mm - saturation_excess_macropores_step_mm;
           // Correct possible mismatch between balance and volume change
           matrix_correction_step_mm = balance_micro_step_mm + Vini_step_micro_mm - Vfin_micro_mm;
           macropore_correction_step_mm = balance_macro_step_mm + Vini_step_macro_mm - Vfin_macro_mm;
@@ -1384,9 +1377,11 @@ NumericVector soilWaterBalance(DataFrame soil, String soilFunctions,
         Psi[l] = Psi_step[l];
         Psi_m[l] = Psi_step_m[l];
         // Rcout << s << " "<< l << " psi " << Psi[l] << " K " << K[l]  << " theta " << Theta[l] <<"\n";
-        S_macro[l] = S_macro_step[l];
-        Kmacro_ms[l] = Kmacro_step_ms[l];
-        theta_macro[l] = theta_macro_step[l];
+        if(soilDomains=="dual") {
+          S_macro[l] = S_macro_step[l];
+          Kmacro_ms[l] = Kmacro_step_ms[l];
+          theta_macro[l] = theta_macro_step[l];
+        }
         theta_micro[l] = theta_micro_step[l];
         // Rcout << s << " "<< l << " th_mi " << theta_micro[l] << " K_mi " << K_ms[l] << " th_ma " << theta_macro[l] << " K_ma " << Kmacro_ms[l]<< " mat_im " << matrixImbibition_m3s[l] << " mat_ex " << matrixExcess_m3s[l] <<"\n";
       }
@@ -1459,6 +1454,124 @@ NumericVector soilWaterBalance(DataFrame soil, String soilFunctions,
                                   _["Substeps"] = (double) total_nsubsteps);
     } 
   }
-
+  
+  return(res);
+}
+//' Soil water balance
+//' 
+//' Function \code{hydrology_soilWaterBalance} estimates water balance of soil layers given water inputs/outputs, including the simulation of water movement within the soil.
+//' 
+//' @param soil Object of class \code{\link{soil}}.
+//' @param soilFunctions Soil water retention curve and conductivity functions, either 'SX' (for Saxton) or 'VG' (for Van Genuchten).
+//' @param rainfallInput Amount of water from rainfall event (after excluding interception), in mm.
+//' @param rainfallIntensity Rainfall intensity, in mm/h.
+//' @param snowmelt Amount of water originated from snow melt, in mm.
+//' @param sourceSink Local source/sink term for each soil layer (from soil evaporation or plant transpiration/redistribution)
+//'        as mm/day.
+//' @param runon Surface water amount running on the target area from upslope (in mm).
+//' @param lateralFlows Lateral source/sink terms for each soil layer (interflow/to from adjacent locations) as mm/day.
+//' @param waterTableDepth Water table depth (in mm). When not missing, capillarity rise will be allowed if lower than total soil depth.
+//' @param infiltrationMode Infiltration model, either "GreenAmpt1911" or "Boughton1989"
+//' @param infiltrationCorrection Correction for saturated conductivity, to account for increased infiltration due to macropore presence
+//' @param soilDomains Either "buckets" (multi-bucket domain), "single" (for single-domain Richards) or "dual" (for dual-permeability model).
+//' @param nsteps Number of time steps per day
+//' @param max_nsubsteps Maximum number of substeps per time step
+//' @param modifySoil Boolean flag to indicate that the input \code{soil} object should be modified during the simulation.
+//' 
+//' @seealso  \code{\link{spwb}}, \code{\link{hydrology_waterInputs}}, \code{\link{hydrology_infiltration}}
+//' 
+//' @details
+//' The multi-bucket model adds/substracts water to each layer and if content is above field capacity the excess percolates to the layer below.
+//' If there is still an excess for the bottom layer, the model will progressively fill upper layers (generating saturation excess if the first layer 
+//' becomes saturated). Every day the some layers are over field capacity, the model simulates deep drainage.
+//' 
+//' The single-domain model simulates water flows by solving Richards's equation using the predictor-corrector method, as described in 
+//' Bonan et al. (2019).
+//' 
+//' The dual-permeability model is an implementation of the model MACRO 5.0 (Jarvis et al. 1991; Larsbo et al. 2005).
+//' 
+//' Both the multi-bucket and the single-domain model can apply a correction to the infiltration rate to account for macroporosity in infiltration. In the
+//' dual-permeability model extra infiltration through macropores is simulated explicitly.
+//' 
+//' @author 
+//' Miquel De \enc{Cáceres}{Caceres} Ainsa, CREAF
+//' 
+//' \enc{María González Sanchís}{Maria Gonzalez Sanchis}, UPV-CTFC
+//' 
+//' @return
+//'   Returns a named vector with different elements, depending on \code{soilDomains}. 
+//'   
+//'   If \code{soilDomains == "buckets"}:
+//'   \itemize{
+//'     \item{\code{Snowmelt}: Snowmelt input (mm).}
+//'     \item{\code{Source/sinks}: Sum of source/sink input across layers (mm).}
+//'     \item{\code{Infiltration}: Water infiltrated into the soil (mm).}
+//'     \item{\code{InfiltrationExcess}: Excess infiltration in the topmost layer (mm) leading to an increase in runoff.}
+//'     \item{\code{SaturationExcess}: Excess saturation in the topmost layer (mm) leading to an increase in runoff.}
+//'     \item{\code{Runoff}: Surface runoff generated by saturation excess or infiltration excess (mm).}
+//'     \item{\code{DeepDrainage}: Water draining from the bottom layer (mm). This quantity is corrected to close the water balance.}
+//'     \item{\code{CapillarityRise}: Water entering the soil via capillarity rise (mm) from the water table, if \code{waterTableDepth} is supplied.}
+//'   }
+//'   If \code{soilDomains == "single"} the named vector contains the following additional elements:
+//'   \itemize{
+//'     \item{\code{Correction}: Amount of water (mm) added to deep drainage to correct the water balance.}
+//'     \item{\code{VolumeChange}: Change in soil water volume (mm).}
+//'     \item{\code{Substep}: Time step of the moisture solving (seconds).}
+//'   }
+//'  If \code{soilDomains == "dual"} the named vector contains the following additional elements:
+//'   \itemize{
+//'     \item{\code{Lateral flows}: Sum of water circulating between micropores and macropores, positive when filling micropores (mm).}
+//'     \item{\code{InfiltrationMatrix}: Water infiltrated into the soil matrix (mm).}
+//'     \item{\code{InfiltrationMacropores}: Water infiltrated into the soil macropore domain (mm).}
+//'     \item{\code{InfiltrationExcessMatrix/InfiltrationExcessMacropores}: Excess infiltration in the topmost layer (mm) leading to an increase in runoff.}
+//'     \item{\code{SaturationExcessMatrix/SaturationExcessMacropores}: Excess saturation in the topmost layer (mm) leading to an increase in runoff.}
+//'     \item{\code{DrainageMatrix}: Water draining from the bottom layer of the matrix domain (mm). This quantity is corrected to close water balance in the micropore domain.}
+//'     \item{\code{DrainageMacropores}: Water draining from the bottom layer of the macropore domain (mm). This quantity is corrected to close the water balance in the macropore domain.}
+//'     \item{\code{CorrectionMatrix}: Amount of water (mm) added to deep drainage of soil matrix to correct the water balance.}
+//'     \item{\code{CorrectionMacropores}: Amount of water (mm) added to deep drainage of macropores to correct the water balance.}
+//'     \item{\code{MatrixVolumeChange}: Change in soil water volume in the soil matrix domain (mm).}
+//'     \item{\code{MacroporeVolumeChange}: Change in soil water volume in the macropore domain (mm).}
+//'   }
+//' 
+//' @references
+//' Bonan, G. (2019). Climate change and terrestrial ecosystem modeling. Cambridge University Press, Cambridge, UK. 
+//' 
+//' Jarvis, N.J., Jansson, P‐E., Dik, P.E. & Messing, I. (1991). Modelling water and solute transport in macroporous soil. I. Model description and sensitivity analysis. Journal of Soil Science, 42, 59–70. 
+//' 
+//' Larsbo, M., Roulier, S., Stenemo, F., Kasteel, R. & Jarvis, N. (2005). An Improved Dual‐Permeability Model of Water Flow and Solute Transport in the Vadose Zone. Vadose Zone Journal, 4, 398–406. 
+//' 
+//' @examples
+//' # Define soil parameters
+//' spar <- defaultSoilParams(4)
+//' 
+//' # Initializes soil hydraulic parameters
+//' examplesoil <- soil(spar)
+//' 
+//' # Water balance in a multi-bucket model
+//' hydrology_soilWaterBalance(examplesoil, "VG", 10, 5, 0, c(-1,-1,-1,-1), 
+//'                            soilDomains = "buckets", modifySoil = FALSE)
+//'                            
+//' # Water balance in a single-domain model (Richards equation)
+//' hydrology_soilWaterBalance(examplesoil, "VG", 10, 5, 0, c(-1,-1,-1,-1), 
+//'                            soilDomains = "single", modifySoil = FALSE)
+//'                     
+//' # Water balance in a dual-permeability model (MACRO)
+//' hydrology_soilWaterBalance(examplesoil, "VG", 10, 5, 0, c(-1,-1,-1,-1), 
+//'                            soilDomains = "dual", modifySoil = FALSE)
+//'   
+//' @name hydrology_soilWaterBalance
+//' @keywords internal
+// [[Rcpp::export("hydrology_soilWaterBalance")]]
+NumericVector soilWaterBalance(DataFrame soil, String soilFunctions, 
+                               double rainfallInput, double rainfallIntensity, double snowmelt, NumericVector sourceSink, 
+                               double runon = 0.0, Nullable<NumericVector> lateralFlows = R_NilValue, double waterTableDepth = NA_REAL,
+                               String infiltrationMode = "GreenAmpt1911", double infiltrationCorrection = 5.0, String soilDomains = "buckets", 
+                               int nsteps = 24, int max_nsubsteps = 3600, bool modifySoil = true) {
+  List SWBcommunication = communicationSoilWaterBalance(soil.nrow());
+  NumericVector res = soilWaterBalance_inner(SWBcommunication, soil, soilFunctions, 
+                                             rainfallInput, rainfallIntensity, snowmelt, sourceSink, 
+                                             runon, lateralFlows, waterTableDepth,
+                                             infiltrationMode, infiltrationCorrection, soilDomains, 
+                                             nsteps, max_nsubsteps, modifySoil);
   return(res);
 }
