@@ -13,7 +13,8 @@ List initSperryNetwork(int c,
                        DataFrame internalWater, DataFrame paramsTranspiration, DataFrame paramsWaterStorage,
                        NumericVector VCroot_kmax, NumericVector VGrhizo_kmax,
                        NumericVector psiSoil, NumericVector VG_n, NumericVector VG_alpha,
-                       double sapFluidityDay = 1.0, List control = NULL) {
+                       List control,
+                       double sapFluidityDay = 1.0) {
   
   NumericVector VCstem_kmax = Rcpp::as<Rcpp::NumericVector>(paramsTranspiration["VCstem_kmax"]);
   NumericVector VCleaf_kmax = Rcpp::as<Rcpp::NumericVector>(paramsTranspiration["VCleaf_kmax"]);
@@ -73,8 +74,8 @@ List initSperryNetworks(List x) {
     networks[c] = initSperryNetwork(c, 
                                     internalWater, paramsTranspiration, paramsWaterStorage,
                                     VCroot_kmax(c,_), VGrhizo_kmax(c,_),
-                                    psiSoil, VG_n, VG_alpha, 1.0, 
-                                    control);
+                                    psiSoil, VG_n, VG_alpha, 
+                                    control, 1.0);
   }
   networks.attr("names") = above.attr("row.names");
   return(networks);
@@ -459,21 +460,6 @@ void innerSperry(List x, List input, List output, int n, double tstep,
   NumericVector minStemPsi = Plants["StemPsi"];
   NumericVector minRootPsi = Plants["RootPsi"];
   
-  // Rcout<<"Leaves\n";
-  List Sunlit = output["SunlitLeaves"];
-  List Shade = output["ShadeLeaves"];
-  NumericVector maxGSW_SL = Sunlit["GSWMax"];
-  NumericVector minGSW_SL = Sunlit["GSWMin"];
-  NumericVector minTemp_SL = Sunlit["TempMin"];
-  NumericVector maxTemp_SL = Sunlit["TempMax"];
-  NumericVector minLeafPsi_SL = Sunlit["LeafPsiMin"];
-  NumericVector maxLeafPsi_SL = Sunlit["LeafPsiMax"];
-  NumericVector maxGSW_SH = Shade["GSWMax"];
-  NumericVector minGSW_SH = Shade["GSWMin"];
-  NumericVector minTemp_SH = Shade["TempMin"];
-  NumericVector maxTemp_SH = Shade["TempMax"];
-  NumericVector minLeafPsi_SH = Shade["LeafPsiMin"];
-  NumericVector maxLeafPsi_SH = Shade["LeafPsiMax"];
   
   // Rcout<<"PlantsInst\n";
   List PlantsInst = output["PlantsInst"];
